# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name SettingsRow
extends PanelContainer

signal reset()

@onready var value_parent = $HBoxContainer/BoxContainer
@onready var label = $HBoxContainer/Label
@onready var reset_button = $HBoxContainer/Reset 

var value_node: Node
var description = "No value was given to the row"
var hide_reset: bool = false

func _ready():
	if hide_reset: reset_button.hide()
	if value_node != null:
		var c: Control = value_node
		c.size_flags_vertical = Control.SIZE_EXPAND_FILL
		c.size_flags_horizontal = Control.SIZE_EXPAND_FILL
		label.text = description
		value_parent.add_child(c)

func set_reset_disabled(d: bool):
	if is_instance_valid(reset_button): reset_button.disabled = d
	else: set_reset_disabled.call_deferred(d)

func _on_reset_pressed():
	reset.emit()
