# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name RangeSetting
extends GameSetting

var min_value: float
var max_value: float
var tick_count
var smooth: bool

func _init(new_id: String, new_default: float, new_min_value: float, new_max_value: float, new_smooth: bool = true, new_tick_count = null):
	super(new_id, new_default)
	min_value = new_min_value
	max_value = new_max_value
	tick_count = new_tick_count
	smooth = new_smooth

func create_row():
	var row = super()
	row.value_node = HSlider.new()
	row.value_node.min_value = min_value
	row.value_node.max_value = max_value
	row.value_node.tick_count = abs(max_value - min_value) if tick_count == null else tick_count
	row.value_node.step = 0 if smooth else (1 if tick_count == null else abs(max_value - min_value) / (tick_count - 1))
	Settings.hook_changed_init(key, row.value_node,
		func(value):
			if is_instance_valid(row):
				row.value_node.value = value
	)
	row.value_node.value_changed.connect(func(value): Settings.write(key, value))
	return row
