# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name Basket
extends Item

var steam: CPUParticles3D = load("res://map/items/steam.tscn").instantiate()
var bubbles: CPUParticles3D = load("res://map/items/deep_frying.tscn").instantiate()

func _init(owned_by_: Node3D):
	super(owned_by_)
	add_child(load("res://map/items/basket.tscn").instantiate())
	base.add_child(steam)
	base.add_child(bubbles)
	base.position.z = -.075

func is_round(): return false

func add_contents(contents: Array[String]):
	for i in contents:
		match i:
			"french-fries":
				base.add_child(load("res://map/items/french_fries_basket.tscn").instantiate())
			"dough":
				base.add_child(load("res://map/items/donut/donut.tscn").instantiate())
			_:
				super([i])

func progress(position_: float, speed: float, warn: bool):
	super(position_, speed, warn)
	steam.emitting = true
	bubbles.emitting = true
	if warn:
		steam.color = Color(.2, .2, .2)
	else:
		steam.color = Color(1.,1.,1.)

func finish():
	super()
	steam.emitting = false
	bubbles.emitting = false

static func base_position() -> Vector3:
	return Vector3(0.,0.015, 0.)
