/*
    Hurry Curry! - a game about cooking
    Copyright (C) 2026 Hurry Curry! Contributors

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

*/
pub mod book_html;
pub mod diagram_svg;

use crate::book_html::render_html_book;
use anyhow::Result;
use clap::{Parser, ValueEnum};
use hurrycurry_data::{book::book, build_gamedata};
use hurrycurry_locale::Locale;
use hurrycurry_protocol::GameConfig;
use std::{fs::File, io::Write, path::PathBuf};

#[derive(Parser)]
struct Args {
    #[arg(long, default_value = "locale")]
    locale_dir: PathBuf,
    #[arg(short, long, default_value = "en")]
    locale: String,
    #[arg(short, long)]
    format: ExportFormat,
    #[arg(short, long, default_value = "5star")]
    map: String,
    #[arg(short, long)]
    output: Option<PathBuf>,
}

#[derive(ValueEnum, Clone, Copy)]
enum ExportFormat {
    Json,
    Html,
}

fn main() -> Result<()> {
    env_logger::init_from_env("LOG");
    let args = Args::parse();

    let (data, serverdata) = build_gamedata(
        "data".as_ref(),
        &GameConfig {
            map: args.map,
            ..Default::default()
        },
        true,
    )?;

    let book = book(&data, &serverdata)?;

    let output = match args.format {
        ExportFormat::Json => serde_json::to_string(&book).unwrap(),
        ExportFormat::Html => {
            let mut locale = Locale::load(&args.locale_dir.join(format!("{}.ini", args.locale)))?;
            locale.merge(Locale::load(&args.locale_dir.join("en.ini"))?);
            render_html_book(&data, &book, &locale)
        }
    };

    if let Some(path) = args.output {
        File::create(path)?.write_all(output.as_bytes())?;
    } else {
        println!("{output}")
    }

    Ok(())
}
