# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name ServerService
extends Service

func name():
	return "Server"

func arguments():
	var args = []
	if Settings.read("server.data_path") != "":
		args.push_back("--data-dir")
		args.push_back(Settings.read("server.data_path"))
	if Settings.read("server.name") != "":
		args.push_back("--server-name")
		args.push_back(Settings.read("server.name"))
	if Settings.read("server.mdns"):
		args.push_back("--mdns")
	if Settings.read("server.upnp"):
		args.push_back("--upnp")
	if Settings.read("server.register"):
		args.push_back("--register")
	args.push_back("--listen")
	args.push_back("%s:%d" % [bind_address(), Settings.read("server.bind_port")])
	return args

func exe_path() -> String:
	var path: String = Settings.read("server.binary_path")
	if path != "": return path
	else: return "hurrycurry-server"

func test_port():
	return Settings.read("server.bind_port")
func test_host():
	return "::1" if Settings.read("server.enable_ipv6") else "127.0.0.1"

static func bind_address() -> String:
	if Settings.read("server.allow_external_connections"):
		return "[::]" if Settings.read("server.enable_ipv6") else "0.0.0.0"
	else:
		return connect_address()

static func connect_address() -> String:
	return "[::1]" if Settings.read("server.enable_ipv6") else "127.0.0.1"
