# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name Burned
extends Item

var steam: CPUParticles3D = load("res://map/items/steam.tscn").instantiate()

func _init(owned_by_: Node3D):
	super(owned_by_)
	steam.color = Color(0., 0., 0.)
	add_child(load("res://map/items/burned.tscn").instantiate())
	base.add_child(steam)


func _ready():
	steam.emitting = true

static func base_position() -> Vector3:
	return Vector3(0., .2, 0.)
