# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name ButtonModel
extends Node3D

func set_up(variant: String):
	match variant:
		"accept":
			$Dynamic/Accept.visible = true
			$Dynamic/Reject.visible = false
		"reject":
			$Dynamic/Accept.visible = false
			$Dynamic/Reject.visible = true
		_:
			push_error("Unrecognized button variant: %s" % variant)
