# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name Pan
extends Item

var steam: CPUParticles3D = load("res://map/items/steam.tscn").instantiate()

func _init(owned_by_: Node3D):
	super(owned_by_)
	add_child(load("res://map/items/pan.tscn").instantiate())
	base.add_child(steam)

func add_contents(contents: Array[String]):
	for i: String in contents:
		if i == "steak" or i == "seared-steak":
			base.add_child(ItemFactory.produce(i, self, .85))
		else:
			super([i])

func progress(position_: float, speed: float, warn: bool):
	super(position_, speed, warn)
	steam.emitting = true
	if warn:
		steam.color = Color(.2, .2, .2)
	else:
		steam.color = Color(1.,1.,1.)
	if sound_id == null:
		sound_id = Sound.item_progress(
			self,
			preload("res://map/items/sounds/frying.ogg"),
			null,
			-5.
		)

func finish():
	super()
	steam.emitting = false

func setup_sounds():
	take_sound.setup([preload("res://map/items/sounds/pot_take.ogg")])
	put_sound.setup([preload("res://map/items/sounds/pot_put.ogg")])

static func base_position() -> Vector3:
	return Vector3(0.,0.015, 0.)
