# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name Glass
extends Item

func _init(owned_by_: Node3D):
	super(owned_by_)
	add_child(load("res://map/items/glass.tscn").instantiate())

func add_contents(contents: Array[String]):
	for i in contents:
		var item: Item
		match i:
			"water": item = GlassFill.new(self, Color(.5, .6, .8))
			"strawberry-shake": item = GlassFill.new(self, Color(.98, .55, .71))
			"tomato-juice": item = GlassFill.new(self, Color(1., .0, .0))
			"strawberry-icecream":
				base.add_child(Icecream.new(self, Color(.98, .55, .71)))
				item = GlassFill.new(self, Color(.98, .55, .71))
			_: push_error("Glass fill not implemented: %s" % contents)
		
		base.add_child(item)

func setup_sounds():
	# TODO: Add custom glass sounds; For now use plate sounds as they are similar
	take_sound.setup([preload("res://map/items/sounds/plate_take.ogg")])
	put_sound.setup([preload("res://map/items/sounds/plate_put.ogg")])

#static func base_position() -> Vector3:
#	return Vector3(0., 0.05, 0.)
