# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends Node3D
class_name ItemRender

var item: Item
var do_rotate: bool

var base := Node3D.new()

func _ready() -> void:
	add_child(base)

func set_item(item_name: String, enable_rotation := true):
	if item != null:
		remove_item()
	
	item = ItemFactory.produce(item_name, base)
	item.animate_spawn()
	do_rotate = enable_rotation
	base.add_child(item)

func remove_item():
	if item != null:
		item.queue_free()

func _process(delta: float):
	if do_rotate:
		base.rotation.y += delta * TAU * .025
