# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name PresetRow
extends GameSetting

var options: Dictionary
var arr: Array[Button]

func _init(id: String, options_: Dictionary):
	super(id)
	options = options_

var prefix = ""
func set_parent(parent):
	super(parent)
	if parent != null: prefix = parent.key

func apply(preset_name: String):
	var preset = options[preset_name]
	for i in preset.keys():
		Settings.write(prefix + "." + i, preset[i])

func hide_reset(): return true

func create_row():
	var row = super()
	row.value_node = HBoxContainer.new()
	for i in options.keys():
		var button := Button.new()
		button.pressed.connect(apply.bind(i))
		button.text = tr(nskey + "." + i)
		row.value_node.add_child(button)
	return row

func load(_d: Dictionary): pass
func save(_d: Dictionary): pass
