# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name DropdownSetting
extends GameSetting

var options: Array
var tooltips: Array

func _init(new_id: String, new_default, new_options: Array, new_tooltips = null):
	super(new_id, new_default)
	options = new_options
	tooltips = new_tooltips if new_tooltips else []

func create_row():
	var row = super()
	row.value_node = OptionButton.new()
	row.value_node.clip_text = true
	for k in options: row.value_node.add_item(tr(nskey + "." + k))
	for i in range(tooltips.size()): row.value_node.set_item_tooltip(i, tooltips[i])
	Settings.hook_changed_init(key, row.value_node,
		func(value):
			if is_instance_valid(row):
				row.value_node.select(options.find(value))
	)
	row.value_node.item_selected.connect(func(item): Settings.write(key, options[item]))
	return row
