# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name Fence
extends WallTile

func _init(ctx: TileFactory.TileCC):
	walls = ["fence"]
	super(ctx)
	var neighboring_fences := ctx.neighbors.map(is_wall).count(true)
	match kind:
		WallKind.STRAIGHT:
			if neighboring_fences == 1:
				base.add_child(load("res://map/tiles/fence_end.tscn").instantiate())
			else:
				base.add_child(load("res://map/tiles/fence_straight.tscn").instantiate())
		WallKind.OUTER_CORNER:
			base.add_child(load("res://map/tiles/fence_corner.tscn").instantiate())
		WallKind.T:
			base.add_child(load("res://map/tiles/fence_t.tscn").instantiate())
		WallKind.CROSS:
			push_warning("no wall cross available!")
			base.add_child(load("res://map/tiles/fence_straight.tscn").instantiate())

static func fm_id() -> String:
	return "grass"
