# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name NumberSetting
extends GameSetting

var placeholder: String
var min_value: int
var max_value: int

func _init(new_id: String, new_default: int, new_min_value: int, new_max_value: int):
	super(new_id, new_default)
	min_value = new_min_value
	max_value = new_max_value

func create_row():
	var row = super()
	var input := SpinBox.new()
	input.min_value = min_value
	input.max_value = max_value

	input.value_changed.connect(func(value): Settings.write(key, value as int))
	Settings.hook_changed_init(key, input,
		func(v):
			if is_instance_valid(input):
				input.value = v
	)
	row.value_node = input
	return row

func load(d: Dictionary):
	if d.has(key): d[key] = int(d[key])
	super(d)
