# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name FoodProcessor
extends Item

var processing: CPUParticles3D = load("res://map/items/processing.tscn").instantiate()

func _init(owned_by_: Node3D):
	super(owned_by_)
	height = 0.8
	add_child(load("res://map/items/food_processor.tscn").instantiate())
	add_child(processing)

func is_round(): return false

func add_contents(contents: Array[String]):
	for i in contents:
		match i:
			"milk":
				add_child(FoodProcessorFill.new(self, Color(1., 1., 1.)))
			"rice-flour":
				add_child(FoodProcessorFill.new(self, Color(1.,1.,.8)))
			"rice":
				processing.color = Color(1.,1.,.8)
				super([i])
			"flour":
				processing.color = Color(.9, .9, .9)
				super([i])
			"mushroom":
				processing.color = Color(0.596, 0.341, 0.259)
				super([i])
			"dough":
				add_child(FoodProcessorFill.new(self, Color8(200, 180, 160)))
			"coconut":
				add_child(FoodProcessorFill.new(self, Color(.8, .5, .4)))
				super([i])
			"strawberry":
				processing.color = Color(.9, .0, .0)
				super([i])
			"strawberry-shake":
				add_child(FoodProcessorFill.new(self, Color8(250, 140, 180)))
			"strawberry-icecream":
				add_child(FoodProcessorFill.new(self, Color8(250, 180, 210)))
			"strawberry-puree":
				add_child(FoodProcessorFill.new(self, Color8(200, 80, 80)))
			"processed-mushroom":
				add_child(FoodProcessorFill.new(self, Color(0.596, 0.341, 0.259)))
			"tomato":
				processing.color = Color(1.,0.,0.)
				super([i])
			"tomato-juice":
				add_child(FoodProcessorFill.new(self, Color(1., .0, .0)))
			_: super([i])

func _process(delta: float):
	super(delta)
	processing.rotation.y += delta * TAU * 10
	base.rotation.y += delta * TAU * 15

func progress(position_: float, speed: float, warn: bool):
	super(position_, speed, warn)
	processing.emitting = true
	if sound_id == null:
		sound_id = Sound.item_progress(
			self,
			preload("res://map/items/sounds/food_processor_running.ogg"),
			preload("res://map/items/sounds/food_processor_stopping.ogg"),
			-10.
		)

func finish():
	super()
	processing.emitting = false

static func base_position() -> Vector3:
	return Vector3(0., 0.4, 0.)
