# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends VBoxContainer
class_name ChatMessage

@onready var fade_away_timer: Timer = $FadeAway
@onready var sender_label: Label = $Sender
@onready var message_label: Label = $MarginContainer/Message

func set_message(username, message: String, username_color: Color, fade_away: bool = false, fade_time: float = 5.):
	if username == null:
		sender_label.hide()
		message_label.add_theme_color_override("font_color", username_color)
	else: 
		sender_label.text = "<%s>" % username
		sender_label.add_theme_color_override("font_color", username_color.lightened(.5))
	message_label.text = message
	if fade_away: fade_away_timer.start(fade_time)

func _on_fade_away_timeout() -> void:
	queue_free()
