# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name HairstylePreview
extends VBoxContainer

signal selected(character_style: Dictionary)

func setup(hairstyle: int, group: ButtonGroup):
	var character_style = Profile.default_profile.character_style.duplicate_deep()
	character_style.hairstyle = hairstyle
	$HairViewport/Node3D/Character.set_style(character_style, "chef")
	$Select.button_group = group
	$Select.text = tr("c.setup.uniform.value").format([hairstyle + 1])
	$Select.pressed.connect(func(): selected.emit(character_style))
