# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name Pizza
extends Item

func _init(owned_by_: Node3D):
	super(owned_by_)

func add_contents(contents: Array[String]):
	base.add_child(load("res://map/items/cooked_cheese_pizza.tscn").instantiate())
	for c: String in contents:
		match c:
			"sliced-cheese": continue
			"tomato-juice": continue
			"sliced-mushroom": base.add_child(load("res://map/items/pizza/mushroom_pizza_topping.tscn").instantiate())
			_: base.add_child(UnknownItem.new(owned_by, c))
