# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#

extends Button
class_name ControllerButton

@export var controller_texture: Texture2D
@export var press_action: String

func _ready():
	set_texture_enabled(Global.using_joypad)
	Global.using_joypad_change.connect(set_texture_enabled)

func _input(_event):
	if press_action == null:
		return
	if not is_visible_in_tree():
		return
	if disabled:
		return
	if Input.is_action_just_pressed(press_action):
		pressed.emit()

func set_texture_enabled(b: bool):
	if b:
		icon = controller_texture
	else:
		icon = null
