# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name KeyboardExplanation
extends DeviceExplanation

const MODULATE_SPEED := 5.

var double_handed := false

@onready var j: Node2D = $VBoxContainer/Control2/JKL/J
@onready var l: Node2D = $VBoxContainer/Control2/JKL/L

@onready var left_hand_label: Label = $VBoxContainer/Control2/JKL/LeftHand/Label
@onready var right_hand: Node2D = $VBoxContainer/Control2/JKL/RightHand
@onready var right_arrow: Sprite2D = $VBoxContainer/Control2/JKL/Sprite2D2

func toggle_double_handed(val: bool) -> void:
	double_handed = val
	right_hand.visible = val
	right_arrow.visible = val
	l.visible = val
	left_hand_label.text = tr("c.controls_explanation.interact_left") if val else tr("c.settings.input.interact")

var t := 0.
func _process(delta: float) -> void:
	if not double_handed: return
	
	t += delta
	var weight: float = (cos(t * MODULATE_SPEED) - 1.) * -0.5
	for i: Node2D in [j,l]:
		i.modulate = Color.WHITE.lerp(Color("ffcd59ff"), weight)
		var size_ := lerpf(1, 1.2, weight)
		i.scale = Vector2(size_, size_)
