# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends Menu

@onready var container = $OuterGap/Panel/InnerGap/VBoxContainer
@onready var outer_gap = $OuterGap

func _ready():
	super()
	var row = Settings.tree.create_row()
	container.add_child(row)
	container.move_child(row, 1)

func _process(_dt):
	var os := OS.get_name()
	if os == "iOS" or os == "Android": return
	# TODO probably bad performance, only update on change
	var margin = max((self.size.x - 1200) / 2, 20)
	outer_gap.add_theme_constant_override("margin_left", margin)
	outer_gap.add_theme_constant_override("margin_right", margin)

func _on_back_pressed():
	exit()

func exit(exit_data = null):
	Settings.save()
	super(exit_data)
