# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends Menu

@onready var message: Label = $Panel/SmartMarginContainer/Contents/Message

func _ready():
	super()
	message.text = data[0]
	previous_path = "res://gui/menus/main/main.tscn"
	Sound.set_music("error", -1, true)

func _on_return_pressed():
	replace_menu("res://gui/menus/main/main.tscn")

func _on_reconnect_pressed() -> void:
	replace_menu("res://gui/menus/game.tscn", data[1])
