# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends Light3D
class_name LightSetup

@export var completely_disable_light_if_shadows_disabled := false
var shadows_enabled: bool

func _ready():
	Settings.hook_changed_init("graphics.shadows", self, apply_settings)

func set_visible_(val: bool):
	if completely_disable_light_if_shadows_disabled and not shadows_enabled:
		return
	visible = val

func apply_settings(setting: bool):
	shadows_enabled = setting
	if completely_disable_light_if_shadows_disabled and not shadows_enabled:
		visible = false
	shadow_enabled = shadows_enabled
