/*
    Hurry Curry! - a game about cooking
    Copyright (C) 2026 Hurry Curry! Contributors

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

*/
use anyhow::{Context, Result, bail};
use hurrycurry_game_core::network::tokio::Network;
use hurrycurry_protocol::PacketC;
use log::info;
use std::{
    collections::BTreeMap,
    net::SocketAddr,
    time::{Duration, Instant},
};
use tokio::{
    net::TcpStream,
    sync::{RwLock, Semaphore},
    time::{sleep, timeout},
};

#[derive(Debug, Clone, Copy)]
struct ConnectTest {
    verified_at: Instant,
    version: (u32, u32),
}

static CONNECT_OK: RwLock<BTreeMap<SocketAddr, ConnectTest>> = RwLock::const_new(BTreeMap::new());
static CONNECT_SEMAPHORE: Semaphore = Semaphore::const_new(10);

pub(crate) async fn prune_connect_cache_task() {
    loop {
        sleep(Duration::from_secs(600)).await;
        CONNECT_OK
            .write()
            .await
            .retain(|_, v| v.verified_at.elapsed().as_secs() < 3600);
    }
}

pub(crate) async fn test_connect(addr: SocketAddr, uri: &str) -> Result<(u32, u32), &'static str> {
    let r = CONNECT_OK.read().await.get(&addr).copied();
    if let Some(r) = r {
        Ok(r.version)
    } else {
        // TODO locks to prevent parallel tests for same addr and dos attempts
        let _permit = CONNECT_SEMAPHORE.acquire().await;
        let res = timeout(Duration::from_secs(10), test_connect_inner(addr, uri))
            .await
            .map_err(|_| "connect timeout")?;
        info!("connect result: {res:?}");
        drop(_permit);

        let version = res.map_err(|_| "server unreachable")?;
        let mut g = CONNECT_OK.write().await;
        g.insert(
            addr,
            ConnectTest {
                verified_at: Instant::now(),
                version,
            },
        );
        Ok(version)
    }
}

async fn test_connect_inner(addr: SocketAddr, uri: &str) -> Result<(u32, u32)> {
    info!("test connect {addr} {uri:?}");
    let stream = TcpStream::connect(addr).await.context("connect")?;
    let net = Network::connect_raw(stream, uri).await.context("upgrade")?;
    let packet = net.receive().await.context("receive")?;
    match packet {
        Some(PacketC::Version { minor, major, .. }) => Ok((major, minor)),
        _ => bail!("bad initial packet"),
    }
}
