# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends Menu

@onready var quit_button = $side/margin/options/first/quit
@onready var margin_container: MarginContainer = $side/margin

func _ready():
	super()
	if OS.has_feature("web"):
		quit_button.hide()
	ServerList.one_shot()
	await get_tree().create_timer(0.7).timeout

func _menu_music(): Sound.set_music("reflets-dans-leau", 3, false)

func _menu_cover(state):
	$side.visible = not state

func exit(exit_data = null):
	Sound.set_music(null)
	super(exit_data)

func _on_quit_pressed():
	quit()

func _on_about_pressed():
	submenu("res://gui/menus/main/about.tscn")

func _on_change_character_pressed():
	replace_menu("res://gui/menus/character.tscn", null, "res://gui/menus/main/main.tscn")

func _on_settings_pressed():
	submenu("res://gui/menus/settings/settings.tscn")

func _on_play_pressed():
	submenu("res://gui/menus/main/play.tscn")
