# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends Menu
class_name ControlsExplanation

var profile_name: String
var using_joypad: bool
var two_handed: bool

@onready var title: Label = $MarginContainer/PanelContainer/SmartMarginContainer/VBoxContainer/Title
@onready var keyboard_explanation: KeyboardExplanation = $MarginContainer/PanelContainer/SmartMarginContainer/VBoxContainer/KeyboardExplanation
@onready var controller_explanation: ControllerExplanation = $MarginContainer/PanelContainer/SmartMarginContainer/VBoxContainer/ControllerExplanation

func _ready():
	profile_name = data[0]
	using_joypad = data[1]
	two_handed = data[2]
	
	@warning_ignore("incompatible_ternary")
	var explanation: DeviceExplanation = controller_explanation if using_joypad else keyboard_explanation
	explanation.visible = true
	explanation.toggle_double_handed(two_handed)
	title.text = tr("c.controls_explanation.two_handed") if two_handed else tr("c.settings.input")
	super()

func _on_accept_pressed() -> void:
	exit()
