# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name MapListItem
extends PanelContainer

@onready var button: Button = $Button

@onready var name_label: Label = $MarginContainer/VBoxContainer/Name
@onready var difficulty_label: Label = $MarginContainer/VBoxContainer/HBoxContainer/Difficulty
@onready var recommended_players_label: Label = $MarginContainer/VBoxContainer/HBoxContainer/RecommendedPlayers

func setup(name_: String, difficulty: int, recommended_players: int):
	name_label.text = name_
	difficulty_label.text = tr("c.map.difficulty.%d" % (difficulty - 1))
	recommended_players_label.text = tr("c.map.players_recommended").format([recommended_players])
