# Hurry Curry! - a game about cooking
# Copyright (C) 2026 Hurry Curry! Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends Control
class_name AnnounceTitle

@onready var init_delay: Timer = $InitialDelay
@onready var prep_timer: Timer = $PreperationTimer
@onready var anim_player: AnimationPlayer = $AnimationPlayer
@onready var label: Label = $Center/Node2D/Label

func announce_start() -> void:
	init_delay.start()
	await init_delay.timeout
	
	Sound.play_game_start()
	show()
	label.text = tr("c.announce.ready")
	anim_player.play("fade_in")
	await anim_player.animation_finished
	prep_timer.start()
	await prep_timer.timeout
	anim_player.play("fade_out")
	await anim_player.animation_finished
	
	label.text = tr("c.announce.go")
	anim_player.play("fade_in")
	await anim_player.animation_finished
	anim_player.play("fade_out")
	await anim_player.animation_finished
	hide()
