/*
    Hurry Curry! - a game about cooking
    Copyright (C) 2026 Hurry Curry! Contributors

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

*/
use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, Serialize)]
pub struct Entry {
    pub name: String,
    pub address: Vec<String>,
    pub players_online: usize,
    pub last_game: i64,
    pub version: (u32, u32),
}

#[derive(Debug, Clone, Deserialize, Serialize)]
pub struct Submission {
    pub secret: u128,
    pub name: String,
    pub players: usize,
    pub last_game: i64,
    pub version: (u32, u32),
    pub uri: String,
}
