/*
    Hurry Curry! - a game about cooking
    Copyright (C) 2026 Hurry Curry! Contributors

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

*/

pub mod diagram_layout;
pub mod recipe_diagram;

use std::cmp::Reverse;

use crate::{
    PrivateGamedata,
    book::{diagram_layout::diagram_layout, recipe_diagram::recipe_diagram},
};
use anyhow::{Context, Result};
use hurrycurry_locale::trm;
use hurrycurry_protocol::{
    Gamedata, Message,
    book::{Book, BookPage},
};

pub fn book(data: &Gamedata, serverdata: &PrivateGamedata) -> Result<Book> {
    let mut pages = Vec::new();

    pages.push(BookPage::Contents {
        title: trm!("b.toc.title"),
        table: vec![],
    });
    let mut toc = Vec::new();

    for (name, rg) in serverdata.recipe_groups.clone() {
        let repr_items = rg
            .repr
            .into_iter()
            .map(|i| data.demands[i.0].input)
            .collect::<Vec<_>>();
        if repr_items.is_empty() {
            continue;
        }

        let mut points = Vec::new();
        for d in rg.demands {
            let demand = &data.demands[d.0];
            points.push((demand.input, demand.points));
        }
        points.sort_by_key(|(_, p)| Reverse(*p));

        let mut diagram = recipe_diagram(data, serverdata, &repr_items)?;
        diagram_layout(&mut diagram).context("during layouting")?;
        let title = Message::Translation {
            id: format!("b.{name}.title"),
            params: vec![],
        };
        toc.push((title.clone(), pages.len()));
        pages.push(BookPage::Recipe {
            title,
            description: Message::Translation {
                id: format!("b.{name}.desc"),
                params: vec![],
            },
            diagram,
            points,
        });
    }

    if let BookPage::Contents { table, .. } = &mut pages[0] {
        *table = toc;
    }
    Ok(Book { pages })
}
